<?php
/**
- 파일: autolinker.php
- 위치: /
- 설명: Autolinker 워드프레스 플러그인의 메인 파일
- 작성일: 2022-xx-xx (이전 날짜로 유지)
- 수정일: 2025-03-07
- 수정내용: DOM 처리 방식 개선으로 싱글 포스트 화면에서 발생하는 오류 수정
 */

/*
Plugin Name: Autolinker
Plugin URI: https://url.kr/p/wp-plugin/autolinker
Description: 관리자가 설정한 키워드에 자동으로 링크를 추가하는 플러그인입니다. 키워드 관리, 클릭 통계 추적, 카테고리/포스트 제외 기능을 제공합니다.
Version: 2.6
Author: 호텔천사
Author URI: https://webworks.co.kr
License: GPL v2
Text Domain: autolinker
*/

// 직접 접근 방지
if (!defined('ABSPATH')) {
    exit;
}

class WP_Autolinker {
	private $items_per_page = 5; // 이 줄 추가
    /**
     * 생성자
     * 필요한 액션과 필터를 등록합니다.
     */
    public function __construct() {
        // 관리자 메뉴 추가
        add_action('admin_menu', array($this, 'add_admin_menu'));
        add_action('admin_menu', array($this, 'add_manual_page'));
        // 컨텐츠 필터 추가
        add_filter('the_content', array($this, 'process_content'));
        // 클릭 추적을 위한 AJAX 처리
        add_action('wp_ajax_log_autolink_click', array($this, 'log_click'));
        add_action('wp_ajax_nopriv_log_autolink_click', array($this, 'log_click'));
        // 스크립트 등록
        add_action('wp_enqueue_scripts', array($this, 'enqueue_scripts'));
        // 관리자 스타일 등록
        add_action('admin_enqueue_scripts', array($this, 'admin_enqueue_scripts'));
        
        // admin-post.php 액션 추가
        add_action('admin_post_autolinker_save_keyword', array($this, 'handle_save_keyword'));
        add_action('admin_post_autolinker_edit_keyword', array($this, 'handle_edit_keyword'));
        add_action('admin_post_autolinker_delete_keyword', array($this, 'handle_delete_keyword'));
    }

    /**
     * 사용설명서 페이지 추가
     */
    public function add_manual_page() {
        add_submenu_page(
            'autolinker-settings',      // 상위 메뉴 슬러그 (Autolinker 메뉴)
            'Autolinker 사용설명서',     // 페이지 제목
            '사용설명서',                // 메뉴 제목
            'manage_options',           // 필요한 권한
            'autolinker-manual',        // 페이지 슬러그
            array($this, 'render_manual_page')  // 콜백 메서드
        );
    }

    /**
     * 사용설명서 페이지 렌더링
     */
    public function render_manual_page() {
        $manual_file = plugin_dir_path(__FILE__) . 'docs/manual.txt';
        if (file_exists($manual_file)) {
            echo '<div class="wrap">';
            echo '<h1>Autolinker 사용설명서</h1>';
            echo '<div style="background: #fff; padding: 20px; margin-top: 20px;">';
            echo '<pre style="white-space: pre-wrap;">';
            echo esc_html(file_get_contents($manual_file));
            echo '</pre>';
            echo '</div>';
            echo '</div>';
        }
    }



    /**
     * 관리자 페이지용 스타일 등록
     */
    public function admin_enqueue_scripts($hook) {
        if (strpos($hook, 'autolinker') !== false) {
            wp_enqueue_style('autolinker-admin', plugins_url('css/admin-style.css', __FILE__));
        }
    }

    /**
     * 프론트엔드 스크립트 등록
     */
    public function enqueue_scripts() {
        wp_enqueue_script('autolinker-tracking', plugins_url('js/tracking.js', __FILE__), array('jquery'), '1.0', true);
        wp_localize_script('autolinker-tracking', 'autolinkerAjax', array(
            'ajaxurl' => admin_url('admin-ajax.php'),
            'nonce' => wp_create_nonce('autolinker-click')
        ));
    }


	
    /**
     * 관리자 메뉴 추가
     */
    public function add_admin_menu() {
        add_menu_page(
            'Autolinker 설정',          // 페이지 제목
            'Autolinker',              // 메뉴 제목
            'manage_options',           // 필요한 권한
            'autolinker-settings',      // 메뉴 슬러그
            array($this, 'render_admin_page'), // 콜백
            'dashicons-admin-links'     // 아이콘
        );

        add_submenu_page(
            'autolinker-settings',      // 상위 메뉴 슬러그
            'Autolinker 설정',          // 페이지 제목
            '설정',                     // 서브메뉴 제목
            'manage_options',           // 필요한 권한
            'autolinker-settings'       // 메뉴 슬러그 (상위와 동일)
        );
    }

    /**
 * 관리자 설정 페이지 렌더링
 */
public function render_admin_page() {
    // 메시지 처리
    if (isset($_GET['message'])) {
        switch ($_GET['message']) {
            case 'updated':
                add_settings_error(
                    'autolinker',
                    'autolinker_updated',
                    '키워드가 성공적으로 수정되었습니다.',
                    'updated'
                );
                break;
            case 'added':
                add_settings_error(
                    'autolinker',
                    'autolinker_added',
                    '키워드가 성공적으로 추가되었습니다.',
                    'updated'
                );
                break;
            case 'deleted':
                add_settings_error(
                    'autolinker',
                    'autolinker_deleted',
                    '키워드가 성공적으로 삭제되었습니다.',
                    'updated'
                );
                break;
            case 'duplicate':
                $keyword = isset($_GET['keyword']) ? urldecode($_GET['keyword']) : '';
                add_settings_error(
                    'autolinker',
                    'autolinker_duplicate',
                    '이미 등록된 키워드입니다: ' . esc_html($keyword),
                    'error'
                );
                break;
            case 'empty_fields':
                add_settings_error(
                    'autolinker',
                    'autolinker_empty',
                    '키워드와 URL은 필수 입력값입니다.',
                    'error'
                );
                break;
            case 'error':
                add_settings_error(
                    'autolinker',
                    'autolinker_error',
                    '처리 중 오류가 발생했습니다.',
                    'error'
                );
                break;
        }
    }

        // 성공/실패 메시지 표시
        settings_errors('autolinker');

		// 검색어 처리
		$search_keyword = isset($_GET['search']) ? sanitize_text_field($_GET['search']) : '';
		
		// 페이징 처리를 위한 현재 페이지 번호
		$current_page = isset($_GET['paged']) ? max(1, intval($_GET['paged'])) : 1;
        
        global $wpdb;

		// 검색 조건이 있는 경우의 WHERE 절
		$where_clause = '';
		$where_params = array();
		if (!empty($search_keyword)) {
			$where_clause = "WHERE keyword LIKE %s";
			$where_params[] = '%' . $wpdb->esc_like($search_keyword) . '%';
		}
		
		// 전체 키워드 수 조회 (검색 조건 적용)
		if (!empty($search_keyword)) {
			$total_items = $wpdb->get_var($wpdb->prepare(
				"SELECT COUNT(*) FROM {$wpdb->prefix}autolinker WHERE keyword LIKE %s",
				'%' . $wpdb->esc_like($search_keyword) . '%'
			));
		
		$total_pages = ceil($total_items / $this->items_per_page);
		
        // 오프셋 계산
        $offset = ($current_page - 1) * $this->items_per_page;

        // 키워드 목록과 통계 데이터를 함께 가져옴 (검색 조건과 페이징 적용)
        $keywords = $wpdb->get_results($wpdb->prepare(
            "SELECT a.*, 
                    COUNT(s.id) as click_count,
                    MAX(s.clicked_at) as last_clicked
             FROM {$wpdb->prefix}autolinker a
             LEFT JOIN {$wpdb->prefix}autolinker_stats s ON a.id = s.keyword_id
             WHERE a.keyword LIKE %s
             GROUP BY a.id
             ORDER BY a.created_at DESC
             LIMIT %d OFFSET %d",
            '%' . $wpdb->esc_like($search_keyword) . '%',
            $this->items_per_page,
            ($current_page - 1) * $this->items_per_page
        ));
    } else {
        $total_items = $wpdb->get_var("SELECT COUNT(*) FROM {$wpdb->prefix}autolinker");
        
        // 검색어가 없을 때는 전체 목록 조회
        $keywords = $wpdb->get_results($wpdb->prepare(
            "SELECT a.*, 
                    COUNT(s.id) as click_count,
                    MAX(s.clicked_at) as last_clicked
             FROM {$wpdb->prefix}autolinker a
             LEFT JOIN {$wpdb->prefix}autolinker_stats s ON a.id = s.keyword_id
             GROUP BY a.id
             ORDER BY a.created_at DESC
             LIMIT %d OFFSET %d",
            $this->items_per_page,
            ($current_page - 1) * $this->items_per_page
        ));
    }
    
    $total_pages = ceil($total_items / $this->items_per_page);
    
    include plugin_dir_path(__FILE__) . 'templates/admin-page.php';
}

    
	/**
     * 키워드 중복 체크 개선
     */
    private function is_keyword_exists($keyword, $exclude_id = null) {
        global $wpdb;
        
        // 대소문자 구분 없이 정확한 키워드 매칭을 위해 BINARY 사용
        $query = $wpdb->prepare(
            "SELECT COUNT(*) FROM {$wpdb->prefix}autolinker WHERE BINARY keyword = %s",
            $keyword
        );
        
        if ($exclude_id !== null) {
            $query = $wpdb->prepare(
                "SELECT COUNT(*) FROM {$wpdb->prefix}autolinker WHERE BINARY keyword = %s AND id != %d",
                $keyword,
                $exclude_id
            );
        }
        
        return (int) $wpdb->get_var($query) > 0;
    }
	
	
	
	 /**
 * 키워드 저장 처리 개선
 */
public function handle_save_keyword() {
    if (!current_user_can('manage_options')) {
        wp_die('권한이 없습니다.');
    }

    if (!isset($_POST['autolinker_nonce']) || 
        !wp_verify_nonce($_POST['autolinker_nonce'], 'autolinker_save_keyword')) {
        wp_die('보안 검증에 실패했습니다.');
    }

    $keyword = sanitize_text_field($_POST['keyword']);
    $url = esc_url_raw($_POST['url']);
    $open_new = isset($_POST['open_new']) ? 1 : 0;

    if (empty($keyword) || empty($url)) {
        wp_safe_redirect(add_query_arg(
            array(
                'page' => 'autolinker-settings',
                'message' => 'empty_fields'
            ),
            admin_url('admin.php')
        ));
        exit;
    }

    try {
        global $wpdb;
        
        // 중복 체크
        if ($this->is_keyword_exists($keyword)) {
            wp_safe_redirect(add_query_arg(
                array(
                    'page' => 'autolinker-settings',
                    'message' => 'duplicate',
                    'keyword' => urlencode($keyword)
                ),
                admin_url('admin.php')
            ));
            exit;
        }
        
        // 키워드 추가
        $result = $wpdb->insert(
            $wpdb->prefix . 'autolinker',
            array(
                'keyword' => $keyword,
                'url' => $url,
                'open_new' => $open_new
            ),
            array('%s', '%s', '%d')
        );

        if ($result === false) {
            wp_safe_redirect(add_query_arg(
                array(
                    'page' => 'autolinker-settings',
                    'message' => 'error'
                ),
                admin_url('admin.php')
            ));
            exit;
        }

        wp_safe_redirect(add_query_arg(
            array(
                'page' => 'autolinker-settings',
                'message' => 'added'
            ),
            admin_url('admin.php')
        ));
        exit;

    } catch (Exception $e) {
        wp_safe_redirect(add_query_arg(
            array(
                'page' => 'autolinker-settings',
                'message' => 'error'
            ),
            admin_url('admin.php')
        ));
        exit;
    }
}



    /**
     * 키워드 수정 처리 개선
     */
    public function handle_edit_keyword() {
        if (!current_user_can('manage_options')) {
            wp_die('권한이 없습니다.');
        }

        if (!isset($_POST['autolinker_nonce']) || 
            !wp_verify_nonce($_POST['autolinker_nonce'], 'autolinker_edit_keyword')) {
            wp_die('보안 검증에 실패했습니다.');
        }

        $keyword_id = isset($_POST['keyword_id']) ? intval($_POST['keyword_id']) : 0;
        $keyword = sanitize_text_field($_POST['keyword']);
        $url = esc_url_raw($_POST['url']);
        $open_new = isset($_POST['open_new']) ? 1 : 0;

        if (empty($keyword) || empty($url)) {
            wp_die('키워드와 URL은 필수 입력값입니다.');
        }

        try {
            global $wpdb;
            
            // 트랜잭션 시작
            $wpdb->query('START TRANSACTION');
            
            // 중복 체크 (현재 키워드는 제외)
            if ($this->is_keyword_exists($keyword, $keyword_id)) {
                throw new Exception('이미 등록된 키워드입니다: ' . esc_html($keyword));
            }
            
            // 키워드 수정
            $result = $wpdb->update(
                $wpdb->prefix . 'autolinker',
                array(
                    'keyword' => $keyword,
                    'url' => $url,
                    'open_new' => $open_new
                ),
                array('id' => $keyword_id),
                array('%s', '%s', '%d'),
                array('%d')
            );

            if ($result === false) {
                throw new Exception('데이터베이스 수정 중 오류가 발생했습니다.');
            }

            // 트랜잭션 커밋
            $wpdb->query('COMMIT');

            // 성공 시 리다이렉트
            wp_safe_redirect(add_query_arg(
                array(
                    'page' => 'autolinker-settings',
                    'message' => 'updated'
                ),
                admin_url('admin.php')
            ));
            exit;

        } catch (Exception $e) {
            // 트랜잭션 롤백
            $wpdb->query('ROLLBACK');
            
            // 오류 메시지 표시
            wp_die($e->getMessage());
        }
    }


    /**
     * 키워드 삭제 처리
     */
    public function handle_delete_keyword() {
        if (!current_user_can('manage_options')) {
            wp_die('권한이 없습니다.');
        }

        if (!isset($_POST['autolinker_delete_nonce']) || 
            !wp_verify_nonce($_POST['autolinker_delete_nonce'], 'autolinker_delete_keyword')) {
            wp_die('보안 검증에 실패했습니다.');
        }

        if (!isset($_POST['keyword_id'])) {
            wp_die('삭제할 키워드 ID가 없습니다.');
        }

        $keyword_id = intval($_POST['keyword_id']);

        global $wpdb;
        $result = $wpdb->delete(
            $wpdb->prefix . 'autolinker',
            array('id' => $keyword_id),
            array('%d')
        );

        if ($result === false) {
            wp_die('데이터베이스 삭제 중 오류가 발생했습니다.');
        }

        wp_safe_redirect(add_query_arg(
            array(
                'page' => 'autolinker-settings',
                'message' => 'deleted'
            ),
            admin_url('admin.php')
        ));
        exit;
    }

/**
 * 컨텐츠 처리 - 개선된 버전
 */
public function process_content($content) {
    // 1. 싱글 포스트 아니거나 내용이 비어있으면 원본 반환
    if (!is_singular() || empty($content)) {
        return $content;
    }

    // 2. 키워드 가져오기
    global $wpdb;
    $keywords = $wpdb->get_results("SELECT * FROM {$wpdb->prefix}autolinker ORDER BY LENGTH(keyword) DESC");
    
    if (empty($keywords)) {
        return $content;
    }
    
    // 3. 안전한 DOM 처리를 위한 설정
    $dom = new DOMDocument();
    libxml_use_internal_errors(true); // DOMDocument 파싱 오류 무시
    
    // UTF-8 인코딩 처리 개선
    $html = '<!DOCTYPE html><html><head><meta charset="UTF-8"></head><body>' . 
            mb_convert_encoding($content, 'HTML-ENTITIES', 'UTF-8') . 
            '</body></html>';
            
    @$dom->loadHTML($html, LIBXML_HTML_NODEFDTD);
    
    // 4. XPath 쿼리 설정
    $xpath = new DOMXPath($dom);
    $textNodes = $xpath->query('//body//text()[not(ancestor::h1) and not(ancestor::h2) and not(ancestor::h3) and not(ancestor::h4) and not(ancestor::h5) and not(ancestor::h6) and not(ancestor::a)]');

    // 5. 한글 조사 및 어미 패턴
    $korean_particles = 
        // 1. 주격/보격 조사
        '이|가|께서|에서|'
        // 2. 목적격 조사
        . '을|를|'
        // 3. 관형격 조사
        . '의|'
        // 4. 부사격 조사
        . '에|에게|한테|께|에서|에서부터|부터|까지|에까지|로|으로|로서|으로서|로써|으로써|처럼|같이|보다|만큼|'
        // 5. 호격 조사
        . '아|야|여|이여|'
        // 6. 공동격 조사
        . '와|과|하고|이랑|랑|'
        // 7. 주제격/보조사
        . '은|는|도|만|까지|조차|마저|부터|'
        // 8. 접속 조사
        . '와|과|이나|나|'
        // 9. 종결 어미
        . '다|이다|요|이에요|예요|에요|군요|네요|는걸요|데요|습니다|입니다|십니다|니다|세요|으세요|'
        // 10. 연결 어미
        . '며|이며|면서|고|이고|라고|이라고|하고|'
        // 11. 명사형 전성 어미
        . '음|기|'
        // 12. 강조 조사
        . '요|'
        // 13. 보조사
        . '도|껏|깨나|꺼나|따라|대로|든|든지|라도|이라도|마다|마저|밖에|부터|뿐|쯤|'
        // 14. 접미사
        . '적|형|성|력|'
        // 15. 특수 조합
        . '이란|란|라는|이라는|라고|이라고|라며|이라며|라서|이라서|라도|이라도|'
        . '만의|뿐만|만을|많이|같은|같은|처럼|같던|같았던|스러운|'
        // 16. 시제 관련
        . '했|였|았|었|겠|되|'
        // 17. 존댓말 관련
        . '시|님|분|'
        // 18. 의문형
        . '인가|인가요|일까|일까요|인지|인지요|될까요|든지|'
        // 19. 부정형
        . '않|못|말|'
        // 20. 추가 패턴
        . '것이|것을|것도|것만|거라|것인|'
        . '인|된|할|하는|하지|해|된다|하며|하면|하게|하다|해서|'
        . '로는|로도|로만|에는|에도|에만|'
        . '씩|째|foo|이야|야|더니|대로|이든|든|자|냐|니|';

    // 각 노드에 변경 필요 여부를 저장할 배열
    $modifications = array();
    
    try {
        // 6. 각 키워드에 대해 처리
        foreach ($keywords as $keyword_index => $keyword) {
            $found = false;
            
            foreach ($textNodes as $node_index => $node) {
                if (!$found && !isset($modifications[$node_index])) {
                    $text = $node->nodeValue;
                    
                    // 정확한 키워드 매칭 (단어+단어 조합의 경우)
                    $exact_pos = mb_strpos($text, $keyword->keyword);
                    
                    // 조사가 붙은 키워드 매칭
                    $pattern = '/(' . preg_quote($keyword->keyword, '/') . ')(' . $korean_particles . ')\b/u';
                    preg_match($pattern, $text, $matches, PREG_OFFSET_CAPTURE);
                    
                    // 정확한 매칭이 있거나, 조사가 붙은 형태로 매칭된 경우
                    if ($exact_pos !== false || !empty($matches)) {
                        $pos = $exact_pos !== false ? $exact_pos : $matches[1][1];
                        $matched_length = $exact_pos !== false ? mb_strlen($keyword->keyword) : mb_strlen($matches[1][0]);
                        
                        // 더 긴 키워드의 일부가 아닌지 확인
                        $is_part_of_longer = false;
                        foreach ($keywords as $longer_keyword) {
                            if ($longer_keyword->keyword !== $keyword->keyword && 
                                mb_strpos($longer_keyword->keyword, $keyword->keyword) !== false &&
                                mb_strpos($text, $longer_keyword->keyword) !== false) {
                                $is_part_of_longer = true;
                                break;
                            }
                        }
                        
                        if (!$is_part_of_longer) {
                            // 텍스트 분할
                            $before = mb_substr($text, 0, $pos);
                            $matched = mb_substr($text, $pos, $matched_length);
                            $after = mb_substr($text, $pos + $matched_length);
                            
                            // 변경 사항 저장
                            $modifications[$node_index] = array(
                                'node' => $node,
                                'before' => $before,
                                'matched' => $matched,
                                'after' => $after,
                                'keyword' => $keyword
                            );
                            
                            $found = true;
                        }
                    }
                }
            }
        }
        
        // 7. DOM 변경 적용 (뒤에서부터 처리하여 DOM 변경에 따른 인덱스 변화 방지)
        krsort($modifications);
        foreach ($modifications as $mod) {
            $node = $mod['node'];
            $before = $mod['before'];
            $matched = $mod['matched'];
            $after = $mod['after'];
            $keyword = $mod['keyword'];
            
            // a. 기존 노드 삭제 전에 부모 노드 가져오기
            $parent = $node->parentNode;
            
            // b. 링크 생성
            $link = $dom->createElement('a');
            $link->setAttribute('href', esc_url($keyword->url));
            $link->setAttribute('class', 'autolink');
            $link->setAttribute('data-keyword-id', $keyword->id);
            if ($keyword->open_new) {
                $link->setAttribute('target', '_blank');
                $link->setAttribute('rel', 'noopener noreferrer');
            }
            $link->appendChild($dom->createTextNode($matched));
            
            // c. 새 노드 생성 및 삽입
            $fragment = $dom->createDocumentFragment();
            
            if ($before) {
                $fragment->appendChild($dom->createTextNode($before));
            }
            
            $fragment->appendChild($link);
            
            if ($after) {
                $fragment->appendChild($dom->createTextNode($after));
            }
            
            // d. 노드 교체 (기존 노드 삭제 및 새 노드 삽입)
            $parent->replaceChild($fragment, $node);
        }
        
        // 8. 결과 HTML 추출 및 반환
        $bodyNode = $dom->getElementsByTagName('body')->item(0);
        if ($bodyNode) {
            $result = '';
            foreach ($bodyNode->childNodes as $childNode) {
                $result .= $dom->saveHTML($childNode);
            }
            
            return $result;
        }
        
        // 실패 시 원본 반환
        return $content;
    }
    catch (Exception $e) {
        // 오류 발생 시 원본 내용 반환
        error_log('Autolinker 오류: ' . $e->getMessage());
        return $content;
    }
    finally {
        // libxml 오류 처리 설정 복원
        libxml_clear_errors();
        libxml_use_internal_errors(false);
    }
}

    /**
     * 클릭 로깅
     */
    public function log_click() {
        check_ajax_referer('autolinker-click');
        
        $keyword_id = intval($_POST['keyword_id']);
        $post_id = intval($_POST['post_id']);
        
        global $wpdb;
        $wpdb->insert($wpdb->prefix . 'autolinker_stats', array(
            'keyword_id' => $keyword_id,
            'post_id' => $post_id,
            'clicked_at' => current_time('mysql')
        ));
        
        wp_die();
    }
}

// 플러그인 인스턴스 생성
$wp_autolinker = new WP_Autolinker();

/**
 * 플러그인 활성화 시 데이터베이스 테이블 생성
 */
register_activation_hook(__FILE__, function() {
    global $wpdb;
    $charset_collate = $wpdb->get_charset_collate();
    
    // wp_autolinker 테이블 생성
    $sql1 = "CREATE TABLE IF NOT EXISTS `{$wpdb->prefix}autolinker` (
        `id` bigint(20) NOT NULL AUTO_INCREMENT,
        `keyword` varchar(255) NOT NULL,
        `url` varchar(255) NOT NULL,
        `open_new` tinyint(1) NOT NULL DEFAULT '0',
        `created_at` timestamp NOT NULL DEFAULT CURRENT_TIMESTAMP,
        `updated_at` timestamp NOT NULL DEFAULT CURRENT_TIMESTAMP ON UPDATE CURRENT_TIMESTAMP,
        PRIMARY KEY (`id`),
        UNIQUE KEY `keyword` (`keyword`)
    ) $charset_collate;";
    
    // wp_autolinker_stats 테이블 생성
    $sql2 = "CREATE TABLE IF NOT EXISTS `{$wpdb->prefix}autolinker_stats` (
        `id` bigint(20) NOT NULL AUTO_INCREMENT,
        `keyword_id` bigint(20) NOT NULL,
        `post_id` bigint(20) NOT NULL,
        `clicked_at` timestamp NOT NULL DEFAULT CURRENT_TIMESTAMP,
        PRIMARY KEY (`id`),
        KEY `keyword_id` (`keyword_id`),
        KEY `post_id` (`post_id`)
    ) $charset_collate;";

    require_once(ABSPATH . 'wp-admin/includes/upgrade.php');
    dbDelta($sql1);
    dbDelta($sql2);
});